function [imc,mout] = motionAutofocus(data,dxa,dya, ...
                                      yorder,kernr,debug)
% MOTIONAUTOFOCUS uses autofocusing to reduce motion artifacts (2D)
%    MOTIONAUTOFOCUS(DATA,DX,DY,YORDER,KERNR,[DEBUG])
%    performs the autofocusing algorithm on a multi-coil dataset,
%    DATA:
%       DX,DY -- [px] matrices that DZA describe 2D linear motion 
%              These matrices contain a set of possible paths. 
%              They all have the size Na x Nm where Na is the total 
%              number of readouts and Nm is the total number of meas.
%       YORDER -- [int] describe the phase-encode ordering. They 
%              have a length of Na. 
%       KERNR -- [px] a 2-element vector that describes the 
%              localization of the motion metric calculation in X 
%              and Y.
%       DEBUG -- optional flag where 0 = debug off, 1 = print time 
%              info, 2+ plot figures in figure(DEBUG).
%
% See also MOTIONCORRECTION, GRADENTROPY, SUMOFSQ, and 
% MOTIONAUTOFOCUS3.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2012
    
% SVN info: 
%   Date:     $Date: 2012-02-14 16:18:31 -0800 (Tue, 14 Feb 2012) $
%   Revision: $Revision: 192 $
%   Author:   $Author: jycheng $
%   Id:       $Id: motionAutofocus.m 192 2012-02-15 00:18:31Z jycheng $
       
    if nargin<8
        debug = 0;
    end
    
    %% Determine filter order for motion metric calculation
    grid = kernr*2+1;
    
    %% Determine data size
    [nx,ny,nc] = size(data);
    %% Motion estimate size
    [npe,nm]      = size(dxa);
    
    %% Init
    out = zeros(size(data));

    %% (I) Recon with no correction (original)
    if debug
        dbdisp('Recon original');
        tic;
    end
    imc = zeros(size(data));
    for C=1:nc
        imc(:,:,C) = ifftnc(data(:,:,C));
    end    
    im_orig = sumofsq(imc);    
    mmetric = gradEntropy(im_orig,grid);
    %% Denote (nCoils+1) as case with no motion
    mout = ones(size(mmetric))+nc; 
    if debug
        dbdisp(sprintf('Done with recon of original: %0.4g seconds',toc));
    end
    
    
    %% (II) Correct images with motion data from each measurement
    if debug
        dbdisp('Correct images with motion data from each coil');
    end
    
    pkx = [-0.5 1/nx];
    pky = [-0.5 1/ny];
    
    %% Loop through all the different motion measurements
    for M=1:nm
        if debug
            tic;
        end
        imc_i = zeros(size(imc));
        %% (1) Loop through all data coils and correct each channel
        for C=1:nc
            %% Extract data    
            data_i = data(:,:,C);  
    
            %% Correction
            data_i =  motionCorrection(data_i,pkx,pky, ...
                                       dxa(:,M),dya(:,M), ...
                                       yorder);        
            %% Transform to Image domain
            imc_i(:,:,C) = ifftnc(data_i);
        end
   
        %% (2) Calculate metric
        mmetric_i = gradEntropy(sumofsq(imc_i),grid);
        
        %% (3) Determine what pixels have an improved motion metric
        mmask = mmetric_i<mmetric;
        
        %% (4) Store
        mout    = mout.*(1-mmask) + M*(mmask);
        mmetric = mmetric.*(1-mmask) + mmetric_i.*(mmask);        
        for C=1:nc
            imc(:,:,C) = imc(:,:,C).*(1-mmask) ...
                + imc_i(:,:,C).*(mmask);
        end
        
        %% (5) Clean-up
        clear data_i imc_i mmetric_i mmask;
    
        if debug
            if debug>1
                im_cor = sumofsq(imc);
                sfigure(debug),   imshow(flipdim([im_orig im_cor],1),[]);
                sfigure(debug+1), imshow(flipdim(mout,1),[]);
                drawnow;
            end            
            dbdisp(sprintf('Finished measurement %d: %g seconds',M,toc));
        end
    end
