function [imc,mout] = motionAutofocus3(data,dxa,dya,dza,...
                                      yorder,zorder,kernr,debug)
% MOTIONAUTOFOCUS3 uses autofocusing to reduce motion artifacts (3D)
%    MOTIONAUTOFOCUS3(DATA,DX,DY,DZ,YORDER,ZORDER,KERNR,[DEBUG])
%    performs the autofocusing algorithm on a multi-coil k-space
%    dataset, DATA:
%       DX,DY,DZ -- [px] matrices that describe 3D linear motion 
%              These matrices contain a set of possible paths. 
%              They all have the size Na x Nm where Na is the total 
%              number of readouts and Nm is the total number of meas.
%       YORDER,ZORDER -- [int] describe the phase/slice encode
%              ordering. They have a length of Na. 
%       KERNR -- [px] a 3-element vector that describes the 
%              localization of the motion metric calculation in the 
%              X,Y, and Z.
%       DEBUG -- optional flag where 0 = debug off, 1 = print time 
%              info, 2+ plot figures in figure(DEBUG).
%    The output to the function is a multi-coil image-domain data.
%
%    [IMC,MOUT] = MOTIONAUTOFOCUS3(...) IMC is the multi-coil image
%    -domain data that is corrected. MOUT is a 3D volumne describing
%    which motion estimate is used to correct each pixel location.
%
% See also MOTIONCORRECTION3, GRADENTROPY, SUMOFSQ, and 
% MOTIONAUTOFOCUS.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2012
    
% SVN info: 
%   Date:     $Date: 2012-02-17 17:06:29 -0800 (Fri, 17 Feb 2012) $
%   Revision: $Revision: 198 $
%   Author:   $Author: jycheng $
%   Id:       $Id: motionAutofocus3.m 198 2012-02-18 01:06:29Z jycheng $
       
    if nargin<8
        debug = 0;
    end
    
    %% Determine filter order for motion metric calculation
    grid = kernr*2+1;
    
    %% Determine data size
    [nx,ny,nz,nc] = size(data);
    %% Motion estimate size
    [npe,nm]      = size(dxa);
    
    %% Init
    out = zeros(size(data));

    %% (I) Recon with no correction (original)
    if debug
        dbdisp('Recon original');
        tic;
    end
    imc = zeros(size(data));
    for C=1:nc
        imc(:,:,:,C) = ifftnc(data(:,:,:,C));
    end    
    im_orig = sumofsq(imc);    
    mmetric = gradEntropy(im_orig,grid);
    %% Denote (nCoils+1) as case with no motion
    mout = ones(size(mmetric))+nm; 
    if debug
        dbdisp(sprintf('Done with recon of original: %0.4g seconds',toc));
    end
    
    
    %% (II) Correct images with motion data from each measurement
    if debug
        dbdisp('Correct images with motion data from each coil');
    end
    
    pkx = [-0.5 1/nx];
    pky = [-0.5 1/ny];
    pkz = [-0.5 1/nz];
    
    %% Loop through all the different motion measurements
    for M=1:nm
        if debug
            tic;
        end
        imc_i = zeros(size(imc));
        %% (1) Loop through all data coils and correct each channel
        for C=1:nc
            %% Extract data    
            data_i = data(:,:,:,C);  
    
            %% Correction
            data_i =  motionCorrection3(data_i,pkx,pky,pkz, ...
                                        dxa(:,M),dya(:,M),dza(:,M), ...
                                        yorder,zorder);        
            %% Transform to Image domain
            imc_i(:,:,:,C) = ifftnc(data_i);
        end
   
        %% (2) Calculate metric
        mmetric_i = gradEntropy(sumofsq(imc_i),grid);
        
        %% (3) Determine what pixels have an improved motion metric
        mmask = mmetric_i<mmetric;
        
        %% (4) Store
        mout    = mout.*(1-mmask) + M*(mmask);
        mmetric = mmetric.*(1-mmask) + mmetric_i.*(mmask);        
        for C=1:nc
            imc(:,:,:,C) = imc(:,:,:,C).*(1-mmask) ...
                + imc_i(:,:,:,C).*(mmask);
        end
        
        %% (5) Clean-up
        clear data_i imc_i mmetric_i mmask;
    
        if debug
            if debug>1
                nz_2 = round(nz/2)+1;
                im_cor = sumofsq(imc);
                sfigure(debug), imshow(flipdim([im_orig(:,:,nz_2) ...
                                                im_cor(:,:,nz_2)],1),[]);
                title(sprintf('Original(left) - Corrected(right): %d',nz_2));
                sfigure(debug+1), imshow(flipdim(mout(:,:,nz_2),1),[]);
                title(sprintf('Motion measurement map: %d',nz_2));
                drawnow;
            end            
            dbdisp(sprintf('Finished measurement %d: %g seconds',M,toc));
        end
    end
