function res = motionCorrection3(data,pkx,pky,pkz,dread,dpe,dse,peorder,slorder)
% MOTIONCORRECTION3 correct for 3D motion
%    MOTIONCORRECTION3(DATA,PKX,PKY,PKZ,DX,DY,DZ,YORDER,ZORDER)
%    corrects for the motion specified in vectors DX, DY, and DZ. 
%    The vectors PKX, PKY, and PKZ describe values of k-space:
%
%       kr = ((1:nro)-1)*PKX(2)+PKX(1); % Readout direction
%       kp = ((1:npe)-1)*PKY(2)+PKY(1); % Phase-encode direction
%       ks = ((1:nse)-1)*PKZ(2)+PKZ(1); % Slice-encode direction
%
%    Motion vectors DX, DY, and DZ all have the same length and 
%    ordered by time. YORDER and ZORDER describes the ordering of 
%    the phase/slice encodes. Note that X is the readout direction, 
%    Y is the phase-encode direction, and Z is the slice-encode 
%    direction.
%
% See also MOTIONAUTOFOCUS and MOTIONCORRECTION.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2012
    
% SVN info: 
%   Date:     $Date: 2012-02-14 16:18:31 -0800 (Tue, 14 Feb 2012) $
%   Revision: $Revision: 192 $
%   Author:   $Author: jycheng $
%   Id:       $Id: motionCorrection3.m 192 2012-02-15 00:18:31Z jycheng $
       
    DEBUG = 0;
    
    if DEBUG
        tic;
    end
    res = zeros(size(data));
        
    nro = size(data,1);
    npe = size(data,2);
    nse = size(data,3);

    kr = ((1:nro)-1)*pkx(2)+pkx(1);
    kp = ((1:npe)-1)*pky(2)+pky(1);
    ks = ((1:nse)-1)*pkz(2)+pkz(1);

    [kpe,kro,kse] = meshgrid(kp, kr, ks); 
                             %[-npe/2:npe/2-1]/npe, ...
                             %[-nro/2:nro/2-1]/nro, ...
                             %[-nse/2:nse/2-1]/nse);    
    
    %% Loop through read-out, rather than phase/slice encodes
    mpo = max(peorder); mso = max(slorder);
    pemat = size(max(peorder),max(slorder));
    for N=1:length(dread(:))
        pemat(peorder(N),slorder(N)) = N;
    end    
    penull = (pemat == 0);
    pemat  = pemat + penull;
    dreadmat = permute(dread(pemat).*(1-penull),[3 1 2]);
    dpemat   = permute(dpe  (pemat).*(1-penull),[3 1 2]);
    dsemat   = permute(dse  (pemat).*(1-penull),[3 1 2]);   
    for N=1:nro
        tmp = data(N,:,:);
        ph  = 2*pi*(kro(N,:,:).*dreadmat + ...
                    kpe(N,:,:).*dpemat   + ...
                    kse(N,:,:).*dsemat);
        tmp = tmp.*exp(i*ph);
        
        res(N,:,:) = tmp;
    end
    
    if DEBUG
        t = toc;        
        dbdisp(sprintf('Done correction: %0.4f s',t));
    end