function varargout = transEstAll(navm,k,res,nc,npe,nref,pfit,filtw,debug)
% TRANSESTALL estimate linear translations for all coils
%    [DXA,DYA,DZA] = TRANSESTALL(NAVM,K,RES,NC,NFR,NREF,PFIT,FILTW,DEBUG) 
%    Estimates displacement by using navigation data in NAVM assuming 
%    rigid translations:
%      NAVM  -- [a.u.] navigation data of size (nNav) x (NC*NFR), where
%                      nNav is the length of the navigator.
%      K     -- [1/cm] k-space trajectory of length nNav
%      RES   -- [mm]   vector of the image resolution 
%      NC    -- [int]  number of coils
%      NFR   -- [int]  number of frames (readouts)
%      PFIT  -- [int]  polynomial fitting to est. any motion drifting 
%                      (default = 1)
%      FILTW -- [int]  filter length to min estimation fluctuations   
%                      (default = 0)
%      DEBUG -- [int]  for debugging purposes: 0 = do nothing, 
%                      1 = print messages, 2+ = plot in figure(DEBUG)
%                      and also print messages (default = 0) 
%     Ouputs [DXA,DYA,DZA] are motion estimates that estimate motion in 
%     terms of number of digital pixels.
%
%     Other uses:
%     [DXA]         = TRANSESTALL(NAVM,K,RES,...)
%     [DXA,DYA]     = TRANSESTALL(NAVM,K,RES,..)
%     [DXA,DYA,...] = TRANSESTALL(NAVM,K,RES,...)
%     For these uses, the number of outputs must correspond with the
%     Length of vector RES. It is also assumed that in the NAVM data
%     the different dimensions are interleaved.
%
%     See also TRANSESTV and TRANSGNW.
% 
% (c) jycheng@mrsrl.stanford.edu 2012
%
    
% SVN info: 
%   Date:     $Date: 2012-02-14 16:18:31 -0800 (Tue, 14 Feb 2012) $
%   Revision: $Revision: 192 $
%   Author:   $Author: jycheng $
%   Id:       $Id: transEstAll.m 192 2012-02-15 00:18:31Z jycheng $

    %% Dimension of data
    ndims = length(res);
    
    if nargout ~= ndims
        dbdisp('WARNING! Dimensions of RES and OUTPUT do not agree');
    end
    
    digkm = zeros(length(k),ndims);
    for D=1:ndims
        %% Convert k-space units from [1/cm] to [1/px]
        digkm(:,D) = k*res(D)/10;
        
        %% Init output
        varargout{D} = zeros(npe,nc);
    end
        
    %% Modify params (step (2) and (3))
    %%  -- for motion estimation, navigation data is not yet interpolated
    %%  -- filter the motion estimate that is not yet interpolated
    nrefi = round(nref/ndims);
    filti = round(filtw/ndims);
    
    %% Init for poly-fitting (step (4)) and interpolation (step (5))
    ta  = (1:npe).';
    tm{1}  = ta(ndims:ndims:end);
    for D=2:ndims
        tm{D} = ta(D-1:ndims:end);
    end

    for C=1:nc
        tic;
        
        navmi = navm(:,(1:npe)+(C-1)*npe);
        
        for D=1:ndims
            %% (1) Extract navigators for one coil
            %%  navigators are ordered as.. y, z, x, y, z, x ...
            %dbdisp('  extract navigators');
            navi = navmi(:,tm{D});
            
            %% (2) Estimate motion
            %dbdisp('  estimate motion');
            dxi = transEstV(navi,digkm(:,D),nrefi);
            
            %% (3) Filter motion (moving average)
            if filti>0
                %dbdisp('  filter motion estimation');
                dxi = smooth(dxi,filti);
            end
            
            %% (4) Remove drifting with polynomial-fitting
            %%   -- Is there a better way?
            if pfit>0
                %dbdisp('  remove drift using poly-fitting');
                px  = polyfit(tm{D},dxi,pfit);
                dxi = dxi-polyval(px,tm{D}(:));
                clear px;
            end
            
            %% (5) Interpolate to total number of frames
            %dbdisp('  interpolate estimation');
            dxi = interp1(tm{D},dxi,ta,'spline');
            
            %% (6) Center estimation with nref as reference point
            %dbdisp('  re-center estimation');
            varargout{D}(:,C) = dxi(:) - dxi(nref);
            
            clear dxi;            
        end % for D(ndims)        
        clear navmi;
                       
        %% For debugging...
        tcalc = toc;
        if debug
            dbdisp(sprintf(['Compute motion estimate for coil %d: %g ' ...
                            'seconds'],C,tcalc));
            if debug>1
                sfigure(debug);
                for D=1:ndims
                    subplot(ndims,1,D); 
                    plot(ta,varargout{D}*res(D)), axis([ta(1) ta(end) -10 10]);
                    ylabel('[mm]'); xlabel('Acquisition number');
                    title(sprintf('Motion in the %d-dimension direction',D));
                end
                drawnow;
            end
        end
   end
