function [dxv] = transEstV(navm,k,nref)
% TRANSESTV estimate linear translation
%    TRANSESTV(SIGM,K,NREF) Estimates displacement by using navigation 
%    data in SIGM assuming a rigid translation. K is the k-space
%    trajectory of the signal. The displacement is compared to time 
%    point NREF. SIGM is a nNav x nFr matrix where nNav is the length
%    of the navigator (also the length of K) and nFr is the number of
%    readout frames. NREF must be less than nFr.
%
%    See also TRANSGNW and TRANSESTALL3.
% 
% (c) jycheng@mrsrl.stanford.edu 2011
%
    
% SVN info: 
%   Date:     $Date: 2012-02-08 16:47:41 -0800 (Wed, 08 Feb 2012) $
%   Revision: $Revision: 1058 $
%   Author:   $Author: jycheng $
%   Id:       $Id: transEstV.m 1058 2012-02-09 00:47:41Z jycheng $
    
    DEBUG_ON = 0;
    
    [Nnav,nV] = size(navm);
    if nref>nV
        dbdisp('Error! Reference point must be within bounds');
        dxv = 0;
        return;
    end
        
    k = k(:);
    dxv = zeros(nV,1);

    %tic;
    sigref = navm(:,1);
    sigref = sigref/sigref(1);
   
    tol_dx1 = 1;
    tol_dc1 = 0.01;
    W2 = abs(k).^.5;    
    W1 = abs(k).^.5;    
        
    for N=2:nV
        sig = navm(:,N);
        
        %% (1) Minimizes phase-variations to help with fitting
        sig = sig/sig(1);
                        
        %% (2) Translation estimation
        %[dx1,dc1] = transGNw(sig,sigref,k,W1,tol_dx1,tol_dc1);
        %[dx2,dc2] = transGNw(sig,sigref.*(exp(-i*2*pi*(k*dx1+dc1))),k,W2);
        %dx = dx1+dx2;
        %dc = dc1+dc2;
        [dx,dc,dm] = transGNw(sig,sigref,k,W2);
        
        %% (3) Store
        dxv(N) = dx;
        
        %% (4) Update reference & average
        sigref = dm*(sigref.*exp(-i*2*pi*(k*dx+dc)));
        sigref = (sigref*(N-1) + sig)/(N);
        
        if DEBUG_ON
            sfigure(4),plot(dxv)
            sfigure(5),plot(cumsum(dxv));
            drawnow;        
        end
    end

    %% Now do the same thing, but backwards
    dxv2 = zeros(N,1);
    for N=nV:-1:1
        sig = navm(:,N);
        
        %% (1)
        sig = sig/sig(1);
                
        %% (2)
        %[dx1,dc1] = transGNw(sig,sigref,k,W1,tol_dx1,tol_dc1);
        %[dx2,dc2] = transGNw(sig,sigref.*(exp(-i*2*pi*(k*dx1+dc1))),k,W2);
        %dx = dx1+dx2;
        %dc = dc1+dc2;
        [dx,dc,dm] = transGNw(sig,sigref,k,W2);

        %% (3)
        dxv2(N) = dx;        
        
        %% (4)
        sigref = dm*(sigref.*exp(-i*2*pi*(k*dx+dc)));        
        sigref = (sigref*(nV-1+(nV-N+1)) + sig)/(nV+nV-N+1);
        
        if DEBUG_ON
            sfigure(6),plot(dxv2)
            sfigure(7),plot(cumsum(dxv2));
            drawnow; 
        end
    end
        
    %% Average both estimations
    dxv = (dxv-dxv2)/2;
    %% Integrate to get final estimation
    dxv = cumtrapz(dxv);
    %% Adjust to a reference point
    dxv = dxv-dxv(nref);
    