function [dx,dc,dm,numit] = transGNw(sig,sigref,k,W,tol_dx,tol_dc,debug)
% TRANSGNW uses Gauss-Newton to calculate the linear translation
%    TRANSGNW(SIG,SIGREF,K,[W]) calculates the difference between 
%    the signals SIG and SIGREF with respect to k-space values K using 
%    the iterative weighted Gauss-Newton algorithm. The optional input 
%    vector W contains the weights used by the algorithm. If W is not
%    specified, than no weights are used. SIG, SIGREF, K, and W should
%    all be 1-D vectors with the same length. The model used  is as 
%    follows: 
%
%       SIG = DM*SIGREF.*exp(i*2*pi*(DX*(K)+DC).
%    
%    Other uses:
%    [DX,DC] = TRANSGNW(SIG,SIGREF,K,[W]) 
%    [DX,DC,DM] = TRANSGNW(SIG,SIGREF,K,[W])
%    [DX,DC,DM,NUMIT] = TRANSGNW(SIG,SIGREF,K,[W])
%    TRANSGNW(SIG,SIGREF,K,[W],[TOL_DX],[TOL_DC],[DEBUG])
%
%   See also TRANSESTV and TRANSESTALL3.
%
% (c) jycheng@mrsrl.stanford.edu 2011.
%

% SVN info: 
%   Date:     $Date: 2012-02-08 16:47:41 -0800 (Wed, 08 Feb 2012) $
%   Revision: $Revision: 1058 $
%   Author:   $Author: jycheng $
%   Id:       $Id: transGNw.m 1058 2012-02-09 00:47:41Z jycheng $

    if nargin<3
        dbdisp('ERROR! Requires at least 3 inputs');
        dx = 0; dm = 0; dc = 0; numit = 0;
        return;
    end
    if nargin<4
        W = ones(size(k));
    end
    if nargin<5 || tol_dx<=0
        tol_dx = 0.01;
    end
    if nargin<6 || tol_dc<=0
        tol_dc = 0.01;
    end
    if nargin<7
        debug = 0;
    end

    %% Do I need this?
    lambda = 1e-20*[1 1e-3]; % [dx dc]; 

    %% Find finite values in signals
    imask = find(isfinite(sig+sigref+k));   
    sigref = sigref(imask);
    sig    = sig(imask)   ;
    k      = k(imask); ki=k;
    W      = W(imask)/norm(W); %% normalize to use same weights    
    
    %% (1) Estimate magnitude difference & update
    dm  = sig(1)/sigref(1);
    dmi = abs(W.*sigref*dm)\abs(W.*sig);
    %% this is equivalent with above:
    %%   dmi = 1/sum(abs(W.*sigref*dm).*abs(W.*sigref*dm))*sum(abs(W.*sigref*dm).*abs(sig));
    dm  = dm*dmi;
    sigref = sigref*dm;

    %% (2) Iteratively estimate phase difference & update
    %% Initialize
    sigrefi = sigref;    
    sigi    = sig;

    %% Variables to solve
    dc  = 0; dci = 1e3;
    dx  = 0; dxi = 1e3;

    if debug>0
        figure(debug);
        subplot(211),plot(k,imag([sigi sigrefi]));
        subplot(212),plot(k,real([sigi sigrefi]));
        legend('sig','sigref');
        pause_prompt;
    end

    %% Used for stopping criteria
    maxit = 60; numit = 0;
    alpha = 1;

    while (abs(dci)>tol_dc || abs(dxi)>tol_dx) && numit < maxit
        
        B = W.*(sigrefi-sigi);
        
        Ax = W.*(-i*2*pi*k.*sigi);
        Ac = W.*(-i*2*pi*sigi);
        
        %% Solve using least-squares 
        Aa =[[real(Ax);imag(Ax)],... %% dx (real)
             [real(Ac);imag(Ac)];... %% dc (real)
             diag(lambda)];
        X = Aa\[real(B);imag(B);0;0];
        
        dxi = X(1); dci = X(2);
        
        %% Update variables
        dx = dx+dxi*alpha;
        dc = dc+dci*alpha;

        %% Update both signals to minimize the change in each
        sigrefi = sigref.*exp( i*pi*(k*dx+dc));
        sigi    = sig   .*exp(-i*pi*(k*dx+dc));

        numit = numit + 1;

        %% For debug use
        if debug>0
            dbdisp(sprintf('increment: dx = %g dc = %g',...
                           dxi,dci));
            figure(debug);
            subplot(211),plot(k,real([sigi sigrefi]));
            subplot(212),plot(k,imag([sigi sigrefi]));
            legend('sig','sigref');
            pause_prompt;
        end
    end
 
    if debug ~= 0
        dbdisp(sprintf('Total iterations: %d',numit));
    end
    
    %% Negate for correction
    dx = -dx;
    dc = -dc;
    return;
